#!/bin/bash
#
# Copyright (C) 2016 CompuLab, Ltd.
# Author: Nikita Kiryanov <nikita@compulab.co.il>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

usage()
{
	cat >&2 <<EOF
Usage: $0 <spl image path> <spl byteswapped image path> <u-boot image path> [-o <path to output file>]

DESCRIPTION
	This script combines the SPL and U-Boot binaries into a single
	cm-t43-firmware image that can be written into on-board SPI flash or SD
	card.

	The spl byteswapped binary is placed at 0K, 64K, and 128K offset in the
	cm-t43-firmware image, a non-byteswapped SPL image is placed at the 192K
	offset, while u-boot is placed in a 256K offset.
	The resulting image should be written in offset 0 of the SPI-flash, or
	offset 320K of an SD-card.
	
OPTIONS
	-o	Normally the script's output is a file called "cm-t43-firmware",
		located in the script's directory. Use this option to provide a
		different output location and filename.
EOF
}

OUTPUT_FILE=cm-t43-firmware

spl_path="$1"
spl_byteswap_path="$2"
uboot_path="$3"
if [[ -z "$spl_path" || -z "$spl_byteswap_path" || -z "$uboot_path" ]] ; then
	usage
	exit 1
fi

shift
shift
shift

if [ "$#" -ge  1 ]; then
	case "$1" in
	-o)
		shift
		OUTPUT_FILE="$1"
		break
		;;
	*)
		echo "Unknown option $1"
		usage
		exit 1
		;;
	esac
fi

echo -e "\nGenerating firmware image from the following binaries:"
echo -e ">>> SPL: `grep -oazE "SPL [0-9]+\.[0-9]+.* \(... +[0-9]+ [0-9]+ - [0-9]+:[0-9]+:[0-9]+\)" $spl_path`"
echo -e ">>> U-Boot: `grep -oazE "U-Boot [0-9]+\.[0-9]+.* \(... +[0-9]+ [0-9]+ - [0-9]+:[0-9]+:[0-9]+.*\)" $uboot_path`\n"
read -p "Press any key to continue or Ctrl-C to abort"

dd if=/dev/zero count=768 bs=1K | tr '\000' '\377' > "$OUTPUT_FILE"
dd if="$spl_byteswap_path" of="$OUTPUT_FILE" bs=1k seek=0 conv=notrunc
dd if="$spl_byteswap_path" of="$OUTPUT_FILE" bs=1k seek=64 conv=notrunc
dd if="$spl_byteswap_path" of="$OUTPUT_FILE" bs=1k seek=128 conv=notrunc
dd if="$spl_path" of="$OUTPUT_FILE" bs=1k seek=192 conv=notrunc
dd if="$uboot_path" of="$OUTPUT_FILE" bs=1k seek=256 conv=notrunc

